library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity LCPU is
	port (
		data:	inout	std_logic_vector(7 downto 0);	-- Memory Data
		addr:	out std_logic_vector(5 downto 0);	-- Memory Address
		we:	out std_logic;								-- Memory Write Enable
		d_reg: out std_logic_vector(7 downto 0);	-- Register
		rst:	in	std_logic;
		clk:	in	std_logic);
end;

architecture CPU_ARCH of LCPU is
	signal	reg:		std_logic_vector(7 downto 0);		-- Register	
	signal 	pc:		std_logic_vector(5 downto 0);		-- Program Counter
	
	type STATE_TYPE is (sfetch, sread, sexec, sload, swriteback, sstore, sjump, swritebackPC);
   signal state : STATE_TYPE;	
begin
	d_reg <= reg;
	process(clk, rst)
	begin
	   if (rst = '0') then 			
			reg	<= (others => '0');	-- Initialize register 
			pc   	<= (others => '0');	-- start execution at memory location 0
			addr	<=	(others => '0');	-- read memory at location 0
			data 	<= (others => 'Z');	-- initialization here, not in ram module!
			state <= sfetch;
	   elsif rising_edge(clk) then
		
			case state is
				when sfetch =>					
					addr	<= pc;										
					we <= '0';
					pc	<= pc + 1;
					state <= sread;
					data <= "ZZZZZZZZ";	-- optional
				when sread =>
					-- read from memory, data is ready next clock
					state <= sexec;
				when sexec =>				
					-- Read data
					case data(7 downto 6) is
						-- Load
						when "00" => 
							we 	<= '0';
							addr 	<= data(5 downto 0);
							state <= sload;
						-- Store
						when "01" => 
							we 	<= '1';					
							addr 	<= data(5 downto 0);
							data 	<= reg;
							state <= sstore;
						-- Add
						when "10" => 							
							reg <= std_logic_vector(unsigned(reg) + unsigned(data(5 downto 0)));
							state <= sfetch;
						-- Jump
						when "11" =>							
							pc 	<= data(5 downto 0);
							state <= sfetch;
						when others => null;
					end case;					
				when sload =>
					-- read from memory, data is ready next clock					
					state <= swriteback;				
				when swriteback =>
					reg 	<= data;		-- Load write back
					state <= sfetch;	-- Fetch Next Instruction				
				when sstore =>
					-- write to memory, data is ready next clock					
					state <= sfetch;
				when others => null;
			end case;			
			
	   end if;
	end process;
	
end CPU_ARCH;
	
