library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity LRAM is
	port ( 
		clk	: in std_logic;
		data	: inout std_logic_vector(7 downto 0):= "ZZZZZZZZ";
		addr  : in std_logic_vector (5 downto 0) := "ZZZZZZ";      
		we    : in std_logic := 'Z';       -- write enable		
		rst	: in std_logic
	);
end;

architecture RAM_ARCH of LRAM is	
	-- memory is 5x8 bit
	-- memory block are ( _ TO _ ) to keep first block first in init
	-- memory bit are ( _ DOWNTO _ ) to keep LSB at right
	type memory is array (0 to 4) of std_logic_vector (7 downto 0);
	signal ram_block : memory;
	shared variable address : integer; -- why shared variable?
begin
	process(clk, rst)
	begin		
		-- TODO: remove reset to syntetize to lpm_ram
	   if (rst = '0') then			
			data 	<= (others => 'Z');			
			-- pre-written bytes into memory
			-- 2 bit istruction op code 6 bit address or immediate value
			ram_block <= (
			"00000100",		-- Load Addr 4
			"10000101", 	-- Add 5
			"01000100", 	-- Store Addr 4
			"11000000",		-- Jump Addr 0			
			"10101010");	-- Addr 4 = 170
		elsif rising_edge(clk) then		
			-- decode address
         address := to_integer(unsigned(addr));
				
			--	Write
			if(we = '1') then
				ram_block(address) <= data(7 downto 0);	-- write data into ram_block
				data <= "ZZZZZZZZ";								-- data is High Z
			elsif(we = '0') then
				-- Read				
				data <= ram_block(address);				
			else 
				-- we High Z
				data <= "ZZZZZZZZ";
			end if;
		
		end if;
	end process;	
end RAM_ARCH;